<?php
require_once 'Unirest.php';

date_default_timezone_set('America/Argentina/Buenos_Aires');

class APIClientBOS
{
   private $m_SolicitudKey;
   private $m_URL;
   private $m_AutorizacionToken;
   private $m_FechaValidez;
   private $m_restClientUrl;
   private $m_headers;
      
   public function __construct($solicitudKey, $URL)
   {
      $this->m_SolicitudKey = $solicitudKey;
      $this->m_URL = $URL;
      
      $this->m_restClientUrl = "http://" . $this->m_URL . "/API/WebServices/";
      $this->m_headers = array('Accept' => 'application/json');
   }
   
   private function autorizacionToken()
   {
      $randomNumerico;
      $fechaSolicitud;
      $sourceMD5;
      $hashMD5;
      $solicitudToken;
      $responseJson;

      //Calculo de 6 digitos random para el token
      $randomNumerico = '';
      for ($x = 0; $x < 6; $x++) 
      {
         $randomNumerico = $randomNumerico . strval(rand(0, 9));
      }
      
      $fechaSolicitud = date('YmdHi');

      //Calculo el hasMD5
      $sourceMD5 = $randomNumerico . $fechaSolicitud . $this->m_SolicitudKey;
      $hashMD5 = md5($sourceMD5);
      
      $solicitudToken = $randomNumerico . $fechaSolicitud . $hashMD5;
      $query = array('solicitudToken' => $solicitudToken);
      
      $response = Unirest\Request::get($this->m_restClientUrl . 'Seguridad/AutorizacionToken.aspx', $this->m_headers, $query);
      
      if ($response->code >= 300 || $response->code < 200)
      {
         throw new Exception('Error al autenticar (EA1): ' .  $response->raw_body);
      }
      
      try
      { 
         $responseJson = $response->body;
         if ($responseJson->autToken !== null)
         {
            $this->m_AutorizacionToken = $responseJson->autToken;     
            $this->m_FechaValidez = DateTime::createFromFormat('Y-m-d\TH:i:s',$responseJson->FechaValidez);
         }
      }
      catch (Exception $e)
      {
         throw new Exception('Error al autenticar (EA2): ' . $e->getMessage());
      }
   }
   
   private function Execute($path, $parametros, $method)
   {
      $responseJson;
      $requestPath;
      $today = new DateTime();
      
      if ($this->m_AutorizacionToken === null && $this->m_FechaValidez <= $today)
      {
         //Autenticacion
         $this->autorizacionToken();
      }
      
      $requestPath = $this->m_restClientUrl . $path . "?AutToken=" . $this->m_AutorizacionToken;
      if ($method == 'GET') {
         $response = Unirest\Request::get($requestPath, $this->m_headers, $parametros);
      }
      else if ($method == 'POST') {
         $response = Unirest\Request::post($requestPath, $this->m_headers, $parametros);
      }
      else if ($method == 'PUT') {
         $response = Unirest\Request::put($requestPath, $this->m_headers, $parametros);
      }
      else if ($method == 'DELETE') {
         $response = Unirest\Request::delete($requestPath, $this->m_headers, $parametros);
      }
      else {
         throw new Exception('Metodo invalido ejecutar el pedido (EP0)');
      }
      
      if ($response->code >= 300 || $response->code < 200)
      {
         throw new Exception('Error al ejecutar el pedido (EP1): ' .  $response->raw_body);
      }

      try
      { 
         $responseJson = $response->body;
      }
      catch (Exception $e)
      {
         throw new Exception('Error al ejecutar el pedido (EP2): ' . $e->getMessage());
      }
      
      return $responseJson;
   }
   
   public function ExecuteGET($path, $parametros)
   {
      return $this->Execute($path, $parametros, 'GET');
   }
   
   public function ExecutePOST($path, $parametros)
   {
      $body = Unirest\Request\Body::json($parametros);
      return $this->Execute($path, $body, 'POST');
   }
   
   public function ExecutePUT($path, $parametros)
   {
      $body = Unirest\Request\Body::json($parametros);
      return $this->Execute($path, $body, 'PUT');
   }
   
   public function ExecuteDELETE($path, $parametros)
   {
      return $this->Execute($path, $parametros, 'DELETE');
   }   
}

?>